<?php
if (!defined("APPLICATION"))
    exit();
/*
Copyright 2008, 2009 Vanilla Forums Inc.
This file is part of Garden.
Garden is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
Garden is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with Garden.  If not, see <http://www.gnu.org/licenses/>.
Contact Vanilla Forums Inc. at support [at] vanillaforums [dot] com
*/

// Define the plugin:
$PluginInfo["Chatwee"] = array(
    "Description" => "Vanilla Chat Plugin by Chatwee lets your site visitors interact with each other in real time via media-rich private, group and public conversations. Customize it to your liking, assign moderators, put various privacy settings in place, and more.",
    "Version" => "2.3.1",
    "RequiredApplications" => array(
        "Vanilla" => "2.0.10"
    ),
    "RequiredTheme" => FALSE,
    "RequiredPlugins" => FALSE,
    "HasLocale" => FALSE,
    "SettingsUrl" => "/plugin/chatwee",
    "SettingsPermission" => "Garden.AdminUser.Only",
    "Author" => "Chatwee",
    "AuthorEmail" => "info@chatwee.com",
    "AuthorUrl" => "https://www.chatwee.com",
    "License" => "GNU GPL2"
);

/**
 *
 *  Chatwee V2 SDK:
 *
 **/

class ChatweeV2_Configuration
{
	private static $_chatId = null;

	private static $_clientKey = null;

	private static $_customUserAgent = null;

	public static function setChatId($chatId) {
		self::$_chatId = $chatId;
	}

	public static function setClientKey($clientKey) {
		self::$_clientKey = $clientKey;
	}

	public static function setCustomUserAgent($customUserAgent) {
		self::$_customUserAgent = $customUserAgent;
	}

	public static function getChatId() {
		return self::$_chatId;
	}

	public static function getClientKey() {
		return self::$_clientKey;
	}

	public static function getCustomUserAgent() {
		return self::$_customUserAgent;
	}

	public static function isConfigurationSet() {
		return self::$_chatId !== null && self::$_clientKey !== null;
	}
}


class ChatweeV2_HttpClient
{
    const API_URL = "http://chatwee-api.com/v2/";

    const USER_AGENT_HEADER = "ChatweeV2 PHP SDK 1.02 RAW";

    private $response;

    private $responseStatus;

    private $responseObject;

    public function get($method, $parameters) {
        if(ChatweeV2_Configuration::isConfigurationSet() === false) {
            throw new Exception("The client credentials are not set");
        }

        $parameters["chatId"] = ChatweeV2_Configuration::getChatId();
        $parameters["clientKey"] = ChatweeV2_Configuration::getClientKey();

        $serializedParameters = self::_serializeParameters($parameters);
        $url = self::API_URL . $method . "?" . $serializedParameters;

        self::call("GET", $url);
    }

    private function call($method, $url) {
        $curl = curl_init();

		$customUserAgent = ChatweeV2_Configuration::getCustomUserAgent();
		$userAgent = $customUserAgent ? $customUserAgent : self::USER_AGENT_HEADER;

        curl_setopt($curl, CURLOPT_TIMEOUT, 60);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_ENCODING, "gzip");
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, Array (
            "Accept: application/xml",
            "Content-Type: application/xml",
            "User-Agent: " . $userAgent
        ));

        $this->response = curl_exec($curl);
        $this->responseStatus = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        $this->responseObject = $this->response ? json_decode($this->response) : null;
        if($this->responseStatus != 200) {
            $responseError = $this->responseObject ? $this->responseObject->errorMessage : "ChatweeV2 PHP SDK unknown error: " . $this->responseStatus;
            $responseError .= " (" . $url . ")";
            throw new Exception($responseError);
        }
    }

    private function _serializeParameters($parameters) {
        if(!is_array($parameters) || count($parameters) == 0) {
            return "";
        }

        $result = "";
        foreach($parameters as $key => $value) {
            $result .= ($key . "=" . ($value ? urlencode(htmlentities($value)) : $value) . "&");
        }

        $result = substr_replace($result, "", -1);
        return $result;
    }

    public function getResponse() {
        return $this->response;
    }

    public function getResponseObject() {
        return $this->responseObject;
    }

    public function getResponseStatus() {
        return $this->responseStatus;
    }
}

class ChatweeV2_Session
{
    private static function getCookieKey() {
        if(ChatweeV2_Configuration::isConfigurationSet() === false) {
            throw new Exception("The client credentials are not set");
        }
        return "chatwee-SID-" . ChatweeV2_Configuration::getChatId();
    }

    public static function getSessionId() {
        $cookieKey = self::getCookieKey();

        return isSet($_COOKIE[$cookieKey]) ? $_COOKIE[$cookieKey] : null;
    }

    public static function setSessionId($sessionId) {
        $hostChunks = explode(".", $_SERVER["HTTP_HOST"]);

        $hostChunks = array_slice($hostChunks, -2);

        $cookieDomain = "." . implode(".", $hostChunks);

        setcookie(self::getCookieKey(), $sessionId, time() + 2592000, "/", $cookieDomain);
    }

    public static function clearSessionId() {
        $hostChunks = explode(".", $_SERVER["HTTP_HOST"]);

        $hostChunks = array_slice($hostChunks, -2);

        $domain = "." . implode(".", $hostChunks);

        setcookie(self::getCookieKey(), "", time() - 1, "/", $domain);
    }

    public static function isSessionSet() {
        return ChatweeV2_Session::getSessionId() !== null;
    }
}

class ChatweeV2_SsoManager {

    public static function registerUser($parameters) {
        if(isSet($parameters["login"]) === false) {
            throw new Exception("login parameter is required");
        }

        $userId = ChatweeV2_SsoUser::register(Array(
            "login" => $parameters["login"],
            "isAdmin" => isSet($parameters["isAdmin"]) === true ? $parameters["isAdmin"] : false,
            "avatar" => isSet($parameters["avatar"]) === true ? $parameters["avatar"] : ""
        ));

        return $userId;
    }

    public static function loginUser($parameters) {
        if(isSet($parameters["userId"]) === false) {
            throw new Exception("userId parameter is required");
        }
        if(self::isLogged() === true) {
            self::logoutUser();
        }

        $sessionId = ChatweeV2_SsoUser::login(Array(
            "userId" => $parameters["userId"],
            "userIp" => ChatweeV2_Utils::getUserIp()
        ));

        ChatweeV2_Session::setSessionId($sessionId);

        return $sessionId;
    }

    public static function logoutUser() {
        if(self::isLogged() === false) {
            return false;
        }
        $sessionId = ChatweeV2_Session::getSessionId();

        ChatweeV2_SsoUser::removeSession(Array(
            "sessionId" => $sessionId
        ));

        ChatweeV2_Session::clearSessionId();
    }

    public static function editUser($parameters) {
        if(isSet($parameters["login"]) === false) {
            throw new Exception("login parameter is required");
        }
        if(isSet($parameters["userId"]) === false) {
            throw new Exception("userId parameter is required");
        }

        $editParameters = Array(
            "userId" => $parameters["userId"],
            "login" => $parameters["login"],
            "avatar" => isSet($parameters["avatar"]) === true ? $parameters["avatar"] : ""
        );

        if(isSet($parameters["isAdmin"]) === true) {
            $editParameters["isAdmin"] = $parameters["isAdmin"];
        }

        ChatweeV2_SsoUser::edit($editParameters);
    }

    private static function isLogged() {
        return ChatweeV2_Session::isSessionSet();
    }
}

class ChatweeV2_SsoUser {

    public static function register($parameters) {
        $requestParameters = Array(
            "login" => $parameters["login"],
            "isAdmin" => $parameters["isAdmin"] === true ? 1 : 0,
            "avatar" => $parameters["avatar"]
        );

        $httpClient = new ChatweeV2_HttpClient();
        $httpClient->get("sso-user/register", $requestParameters);

        $userId = $httpClient->getResponseObject();

        return $userId;
    }

    public static function login($parameters) {
        $requestParameters = Array(
            "userId" => $parameters["userId"]
        );

		if(isSet($parameters["userIp"])) {
			if(filter_var($parameters["userIp"], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) !== false) {
				$requestParameters["userIp"] = $parameters["userIp"];
			}
		}

        $httpClient = new ChatweeV2_HttpClient();
        $httpClient->get("sso-user/login", $requestParameters);

        $sessionId = $httpClient->getResponseObject();

        return $sessionId;
    }

    public static function removeSession($parameters) {
        $requestParameters = Array(
            "sessionId" => $parameters["sessionId"]
        );

        $httpClient = new ChatweeV2_HttpClient();
        $httpClient->get("sso-user/remove-session", $requestParameters);
    }

	public static function validateSession($parameters) {
		$requestParameters = Array(
			"sessionId" => $parameters["sessionId"]
		);

		$httpClient = new ChatweeV2_HttpClient();
		$httpClient->get("sso-user/validate-session", $requestParameters);

		$validationResponse = $httpClient->getResponseObject();

		return $validationResponse;
	}

    public static function logout($parameters) {
        $requestParameters = Array(
            "userId" => $parameters["userId"]
        );

        $httpClient = new ChatweeV2_HttpClient();
        $httpClient->get("sso-user/logout", $requestParameters);
    }

    public static function edit($parameters) {
        $requestParameters = Array(
            "userId" => $parameters["userId"],
            "login" => $parameters["login"],
            "avatar" => $parameters["avatar"]
        );

        if(isSet($parameters["isAdmin"]) === true) {
            $requestParameters["isAdmin"] = $parameters["isAdmin"] === true ? 1 : 0;
        }

        $httpClient = new ChatweeV2_HttpClient();
        $httpClient->get("sso-user/edit", $requestParameters);
    }
}

class ChatweeV2_Utils {

    public static function getUserIp() {
        if (!empty($_SERVER["HTTP_CLIENT_IP"])) {
            $ip = $_SERVER["HTTP_CLIENT_IP"];
        } elseif (!empty($_SERVER["HTTP_X_FORWARDED_FOR"])) {
            $ip = $_SERVER["HTTP_X_FORWARDED_FOR"];
        } else {
            $ip = $_SERVER["REMOTE_ADDR"];
        }
        return $ip;
    }
}

class ChatweePlugin extends Gdn_Plugin
{
    public function __construct() {
        $this->config = C("Plugin.Chatwee");
    }

    public function EntryController_AuthSuccess_Handler($Sender) {
        $Session = Gdn::Session();
        if ($Session->IsValid()) {
            $Sender->InformMessage('You are already signed in as ' . $Session->User->Name . '.');
        }
    }

    public function PluginController_Chatwee_Create($Sender) {

        $Sender->Title("Vanilla Chat Plugin by Chatwee");
        $Sender->AddSideMenu("plugin/chatwee");

        $Sender->Form = new Gdn_Form();
        $this->Dispatch($Sender, $Sender->RequestArgs);

    }

    public function Controller_Index($Sender) {

        $Sender->Permission("Vanilla.Settings.Manage");

        $Sender->SetData("PluginDescription", $this->GetPluginKey("Description"));

		$Validation = new Gdn_Validation();

		$ConfigurationModel = new Gdn_ConfigurationModel($Validation);
		$ConfigurationModel->SetField("Plugin.Chatwee.code");
		$ConfigurationModel->SetField("Plugin.Chatwee.sso_enable");
        $ConfigurationModel->SetField("Plugin.Chatwee.chat_id");
        $ConfigurationModel->SetField("Plugin.Chatwee.api_key");
        $ConfigurationModel->SetField("Plugin.Chatwee.moderators");
		$Sender->Form->SetModel($ConfigurationModel);

		if ($Sender->Form->AuthenticatedPostBack() === FALSE) {
			$Sender->Form->SetData($ConfigurationModel->Data);
		} else {
        	$Data = $Sender->Form->FormValues();

        	if ($Sender->Form->Save() !== FALSE)
        		$Sender->StatusMessage = T("Your settings have been saved.");
		}

        $Sender->Render($this->GetView("chatwee.php"));
    }

    public function Base_GetAppSettingsMenuItems_Handler($Sender) {

        $Menu =& $Sender->EventArguments["SideMenu"];
        $Menu->AddLink("Add-ons", "Chatwee", "plugin/chatwee", "Garden.AdminUser.Only");
    }

    public function Setup() {
        Gdn::structure()
           ->table("chatwee_users_data")
           ->column("user_id", "varchar(50)", FALSE, "primary")
           ->column("chatwee_id", "varchar(80)", FALSE)
           ->set();
    }

    public function Base_Render_Before($Sender) {

    	if($this->config["sso_enable"] == 1) {
    		$this->synchronize_sso_session();
    	}

    	$chatweeCode = C("Plugin.Chatwee.code", 100);

        if($chatweeCode) {
            $Sender->Head->AddString($chatweeCode);
            $Sender->setHeader('X-XSS-Protection','0');
        }
    }

    public function Base_AfterSignIn_Handler($Sender) {

    	if($this->config["sso_enable"] == 1) {
        	$this->sso_login();
        }
    }

    public function EntryController_SignOut_Handler($Sender) {

    	if($this->config["sso_enable"] == 1) {
        	$this->sso_logout();
        }
    }

    public function log_error($error) {
    	LogMessage(__FILE__, __LINE__, 'ChatweePlugin', 'log_error', $error);
    }

    public function set_sso_credentials() {

        ChatweeV2_Configuration::setChatId($this->config["chat_id"]);
        ChatweeV2_Configuration::setClientKey($this->config["api_key"]);
        ChatweeV2_Configuration::setCustomUserAgent("Chatwee Vanilla Plugin 2.3.1");
    }

    public function synchronize_sso_session() {

		if(!empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && strtolower($_SERVER["HTTP_X_REQUESTED_WITH"]) === "xmlhttprequest") {
			return false;
		}

		$user = Gdn::Session()->User;
		$is_user_logged_in = $user->UserID ? true : false;

		$this->set_sso_credentials();

		if($is_user_logged_in === true && ChatweeV2_Session::isSessionSet() === false) {
			$this->sso_login();
		}
		if($is_user_logged_in === true && ChatweeV2_Session::isSessionSet() === true) {
			try {
				$is_session_valid = ChatweeV2_SsoUser::validateSession(array(
					'sessionId' => ChatweeV2_Session::getSessionId(),
				));
				if($is_session_valid === false)	{
					$this->sso_login();
				}
			} catch(Exception $exception) {
				$this->log_error("Error while executing synchronize_sso_session: " . $exception->getMessage());
			}
		}
    }

    public function sso_login() {

    	if(!$this->config["chat_id"] || !$this->config["api_key"]) {
    		return false;
    	}

    	$this->set_sso_credentials();

    	$user = Gdn::Session()->User;
    	$user_id = $user->UserID;

        $user_chatwee_id = $this->fetch_user_chatwee_id($user_id);

        if (!$user_chatwee_id) {
            $user_chatwee_id = $this->sso_register($user);
        }

		try	{
			ChatweeV2_SsoUser::edit(Array(
				"userId" => $user_chatwee_id,
				"login" => $user->Name,
				"avatar" => $user->Photo != "" ? Gdn_Upload::Url(ChangeBasename($user->Photo, "p%s")) : "",
				"isAdmin" => $this->is_chatwee_admin($user)
			));
		} catch(Exception $exception) {
			if(strpos($exception->getMessage(), "User does not exist") !== false) {
				$user_chatwee_id = $this->sso_register($user);
			} else {
				$this->log_error("Error while executing sso_login: " . $exception->getMessage());
			}
		}

		try {
			ChatweeV2_SsoManager::loginUser(Array(
				"userId" => $user_chatwee_id
			));
		} catch(Exception $exception) {
			$this->log_error("Error while executing sso_login: " . $exception->getMessage());
		}
    }

    public function sso_logout() {

        $this->set_sso_credentials();

        try {
            ChatweeV2_SsoManager::logoutUser();
        } catch(Exception $exception) {
            $this->log_error("Error while executing sso_logout: " . $exception->getMessage());
        }
    }

    public function fetch_user_chatwee_id($user_id) {

        $row = Gdn::sql()->getWhere("chatwee_users_data", array("user_id" => $user_id))->firstRow(DATASET_TYPE_ARRAY);
        return $row ? $row["chatwee_id"] : null;
    }

    public function sso_register($user) {

        $user_id = $user->UserID;

        try {

            $chatwee_user_id = ChatweeV2_SsoManager::registerUser(Array(
                "login" => $user->Name,
                "avatar" => $user->Photo != "" ? Gdn_Upload::Url(ChangeBasename($user->Photo, "p%s")) : "",
                "isAdmin" => $this->is_chatwee_admin($user)
            ));

            Gdn::sql()->delete("chatwee_users_data", ["user_id" => $user_id]);

	        Gdn::sql()->insert("chatwee_users_data", array(
	           "user_id" => $user_id,
	           "chatwee_id" => $chatwee_user_id
	        ));

            return $chatwee_user_id;

        } catch(Exception $exception) {
            $this->log_error("Error while executing sso_register: " . $exception->getMessage());
            return null;
        }
    }

    public function is_chatwee_admin($user) {

        if($user->Admin == 1) {
            return TRUE;
        }

        $user_id = $user->UserID;
        $row = Gdn::sql()->getWhere("UserRole", array("RoleID" => 32, "UserID" => $user_id))->firstRow(DATASET_TYPE_ARRAY);

        if ($row && $this->config["moderators"] == 1) {
            return TRUE;
        }

        return FALSE;
    }

}
